/**
 * \file gre/hypermgr.h
 * \brief GRE HyperIndex Link Manager definitions.
 *
 * \if NODOC
 * $Id: hypermgr.h_v 1.19 2011/10/20 12:58:40 mju Exp $
 *
 * $Log: hypermgr.h_v $
 * Revision 1.19  2011/10/20 12:58:40  mju
 * Remove redundant/unneeded hdrs.
 *
 * Revision 1.18  2008/01/09 21:41:42  mju
 * Add drawShapes with group or layer.
 *
 * Revision 1.15  2006/06/15 16:33:53  scowan
 * Uses changes to hyper index interface classes.
 * Moved all functions into hyperidxmgr class.
 *
 * Revision 1.6  2003/11/12 13:21:29  mju
 * Add HyperShapeTypeToElemType.
 *
 * Revision 1.3  2003/05/14 21:24:21  mju
 * Add HasLinks using viewable.
 *
 * Revision 1.1  2003/04/30 21:40:24  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_HYPERMGR_H
#define	INC_GRE_HYPERMGR_H

#ifndef	INC_GRE_BASE
	#include <gre/base.h>
#endif

#ifndef	INC_RVC_HYPERIDX_H
	#include <rvc/hyperidx.h>
#endif

#ifndef	INC_MI32_POLYLINE_H
	#include <mi32/polyline.h>
#endif

//===================================================================================================================

namespace GRE {

class HYPERLINK : public RVC::HYPERINDEX::LINK {
	public:
		HYPERLINK (
			) :
			LINK(),
			m_ShapeNum(0),
			m_Layer(0)
			{}
			
		void Clear (
			) {
			m_ShapeNum = 0;
			m_Layer = 0;
			LINK::Clear();
			return;
			}
			
		LAYER* GetLayer (
			) const { return (m_Layer); }
			
		UINT32 GetShapeNum (
			) const { return (m_ShapeNum); }
			
		void SetLayer (
			LAYER* layer
			) { m_Layer = layer; }
			
		void SetShapeNum (
			UINT32 ShapeNum
			) { m_ShapeNum = ShapeNum; }
			
	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		UINT32 m_ShapeNum;						// Shape number that link is referenced by
		LAYER *m_Layer;							// Layer that this link is referenced by
		#endif
	};
	

class HYPERSHAPE : public RVC::HYPERINDEX::SHAPE {
	public:
	
		HYPERSHAPE (
			) :
			SHAPE(),
			m_ShapeNum(-1),
			m_Highlighted(false)
			{}
			
		void Clear (
			) {
			m_Points.Clear();
			m_Links.Clear();
			m_Highlighted = false;
			SetExtents(DRECT3D());
			return;
			}
			
		ERRVALUE DeleteShape (
			RVC::HYPERINDEX& HyperObj
			);
			
		const MILIST<GRE::HYPERLINK>& GetLinks (
			) const { return (m_Links); }
			
		const POLYLINE& GetPoints (
			) const { return (m_Points); }
			
		bool IsHighlighted (
			) const { return (m_Highlighted); }
			
		ERRVALUE ReadShape (
			RVC::HYPERINDEX& HyperObj,
			INT32 ShapeNum,
			GRE::LAYER *layer
			);
		
		void SetHighlighted (
			bool value
			) { m_Highlighted = value; }
			
		void SetLinks (
			const MILIST<GRE::HYPERLINK>& Links
			) { m_Links = Links; }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_ShapeNum;
		bool m_Highlighted;								//!< Set if shape is currently highlighted
		POLYLINE m_Points;								//!< Array of points if appropriate
		MILIST<GRE::HYPERLINK> m_Links;				//!< Array of links
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


class HYPERIDXMGR {
	public:
		enum FINDFLAGS {
			FINDFLAG_Default =			0x00,
			FINDFLAG_SearchAllElem =	0x01,
			FINDFLAG_FirstOnly =			0x02
			};

		//! Draw specified HyperIndex shape for layer.
		static ERRVALUE DrawShape (
			VIEW *view, 
			LAYER *layer, 
			int obj, 
			const TRANS2D_MAPGEN& LayerToDisp,
			INT32 shapenum, 
			UINT32 drawflags
			);

		//! Draw specified HyperIndex shape for layer.
		static ERRVALUE DrawShape (
			VIEW *view, 
			LAYER *layer, 
			int obj, 
			INT32 shapenum, 
			UINT32 drawflags
			);

		//! Draw all HyperIndex link shapes for view.
		static int DrawShapes (
			VIEW *view, 
			UINT32 drawflags
			);

		//! Draw all HyperIndex link shapes for group in view.
		static int DrawShapes (
			VIEW *view,
			const GRE::GROUP *group,
			UINT32 drawflags
			);

		//! Draw all HyperIndex link shapes for layer in view.
		static int DrawShapes (
			VIEW *view,
			GRE::LAYER *layer,
			UINT32 drawflags
			);

		//! Find shape corresponding to given layer element.
		//!
		//! @return Shape number, -1 if no shape, < -1 if error.
		static int FindLayerElemShape (
			LAYER *layer, 
			RVC::HYPERINDEX::SHAPE::TYPE ShapeType,
			INT32 ElemNum
			);

		//! Return list of shape numbers for given layer object.
		//!
		//! @return Number of shapes or error < 0.
		static int FindLayerObjShapes (
			VIEW *view, 
			LAYER *layer, 
			int obj, 									//!< Layer object number
			const DPOINT2D& lpoint, 				//!< Point in layer's object coordinates 
			SIMPLE_ARRAY<INT32>& ShapeList,		//!< Shape numbers (within HyperIndex object) returned
			FINDFLAGS flags = FINDFLAG_Default
			);

		//! Find hyper links for view given point.
		static int FindLinks (
			VIEW *view, 
			const DPOINT2D& spoint, 				//!< Point in "screen" coordinates
			MILIST<HYPERLINK>& linklist,
			FINDFLAGS flags = FINDFLAG_Default
			);

		//! Find hyper links for view given point.
		static int FindLinksView (
			VIEW *view, 
			const DPOINT2D& lpoint, 				//!< Point in "view" coordinates
			MILIST<HYPERLINK>& linklist,
			FINDFLAGS flags = FINDFLAG_Default
			);

		//! Find neighbor links/shapes for given layout.
		static int FindNeighbors (
			LAYOUT *layout,							//!< Layout, NULL if point is in Lat-Lon
			LAYOUT *playout, 
			const DPOINT2D& point, 						//!< Point in layout or Lat-Lon coordinates
			const FILEPATH& parentfilename, 			//!< Name of parent file (previous navigation step)
			const RVC::OBJECTNAME& parentobjname, 	//!< Name of parent object
			const FILEPATH& curfilename, 				//!< Name of current file to exclude from neighbor list
			const RVC::OBJECTNAME& curobjname, 		//!< Name of current object to exclude from neighbor list
			HYPERLINK neighbors[8]				//!< Array of neighbors returned
			);

		//! Return HyperIndex object handle for specified layer object.
		static ERRVALUE GetHyperObjItem (
			LAYER *layer, 
			int obj,
			RVC::OBJITEM& HyperObjItem
			);

		//! Get shape list for specified layer.
		static int GetLayerObjShapeList (
			LAYER *layer, 
			int obj, 
			MILIST<HYPERSHAPE>& ShapeList
			);

		//! Determine if has any hyperlinks for specified viewable.
		static bool HasLinks (
			VIEWABLE *viewable
			);

		//! Determine if has any hyperlinks for specified point.
		static bool HasLinks (
			VIEW *view, 
			const DPOINT2D& spoint, 				//!< Point in "screen" coordinates
			FINDFLAGS flags = FINDFLAG_Default
			);

		static void Init (
			);
			
		//! Open HyperIndex object for specified layer object.
		//! If one does not exist it will be created if OpenMode == OPENMODE_Write
		//!
		//! @return TRUE if opened file, FALSE if file was already open, < 0 if error.
		static ERRVALUE OpenHyperObject (
			LAYER *layer, 
			int obj, 
			RVC::HYPERINDEX& HyperObj,
			RVC::OBJECT::OPENMODE OpenMode
			);

		//! Add HyperIndex shape to array of shapes for layer.
		static int ShapeAddToLayer (
			LAYER *layer, 
			int obj, 									//!< Layer object
			RVC::HYPERINDEX& HyperObj,
			INT32 shapenum
			);

		//! Delete HyperIndex shape from layer.
		static int ShapeDelete (
			LAYER *layer, 
			int obj, 									//!< Layer object
			RVC::HYPERINDEX& HyperObj,
			INT32 shapenum
			);

		//! Determine ELEMTYPE for HyperIndex ShapeType (HYPERSHAPE...)
		//! @return Element type, ELEMTYPE_Empty if can't convert.
		static ELEMTYPE ShapeTypeToElemType (
			LAYER *layer,							//!< Layer
			RVC::HYPERINDEX::SHAPE::TYPE ShapeType							//!< HYPERSHAPE_... shape type
			);

		//! Update after any change to shape (except deletion)/
		static int ShapeUpdate (
			LAYER *layer, 
			int obj, 									//!< Layer object
			RVC::HYPERINDEX& HyperObj,
			INT32 shapenum
			);

	};
#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(HYPERIDXMGR::FINDFLAGS);
#endif


}	// End namespace GRE

//===================================================================================================================
//	Legacy definitions - Begin

#define	HYPERDRAWFLAG_Highlight				0x01
#define	HYPERDRAWFLAG_Unhighlight			0x02
#define	HYPERDRAWFLAG_ShowTransparent		0x04
#define	HYPERDRAWFLAG_NoDraw					0x08

//===================================================================================================================

#endif	// INC_GRE_HYPERMGR_H
