/**
 * \file mgui/toolpolyline.h
 * \brief MGUI::TOOL_POLYLINE class definition
 *
 * \if NODOC
 * $Id: toolpolyline.h_v 1.38 2011/12/22 18:02:16 mju Exp $
 *
 * $Log: toolpolyline.h_v $
 * Revision 1.38  2011/12/22 18:02:16  mju
 * Rename method to clarify intent.
 *
 * Revision 1.37  2011/12/21 22:30:35  mju
 * Don't use drawing context directly so can handle stereo.
 *
 * Revision 1.36  2011/12/09 22:29:16  mju
 * No dc in drawMarkerStart/End.
 *
 * Revision 1.35  2011/12/09 20:23:26  mju
 * No context in v_draw.
 *
 * Revision 1.34  2011/11/08 17:47:57  bcolombe
 * Add v_OnPointerLeave
 *
 * Revision 1.33  2011/06/21 14:13:24  mju
 * *** empty log message ***
 *
 * Revision 1.32  2010/11/23 18:48:24  mju
 * Add parm for computePointerZone for whether to draw marker.
 *
 * Revision 1.31  2008/10/22 14:08:29  mju
 * Add updateDisplay parm to resetLastDrawnSection.
 *
 * Revision 1.30  2008/10/17 21:09:30  mju
 * Keep polyline of last-drawn points so can adjust filter settings.
 * Remove unused timing members.
 *
 * Revision 1.29  2008/08/22 22:47:33  scowan
 * Added timer array.
 *
 * Revision 1.28  2008/08/11 17:55:30  scowan
 * Added parameter for filter items.
 *
 * Revision 1.27  2008/07/15 16:31:01  mju
 * Add addLineIfNoActive.
 *
 * Revision 1.26  2008/07/11 22:47:06  scowan
 * Changes to z value assignments.
 *
 * Revision 1.25  2008/07/09 22:32:03  scowan
 * Esc and more degrees of movement for <shift> key.
 *
 * Revision 1.24  2008/05/29 16:25:34  scowan
 * Added isClosed method.
 *
 * Revision 1.23  2008/05/22 17:58:53  scowan
 * Added method to draw cursor marker.
 *
 * Revision 1.22  2008/05/21 22:35:33  scowan
 * Added updateActiveLine.
 *
 * Revision 1.21  2008/05/21 19:37:33  scowan
 * Added updateActiveLine.
 *
 * Revision 1.20  2008/05/13 21:10:36  scowan
 * Erase cursor markers when changing lines.
 *
 * Revision 1.19  2008/05/09 22:56:29  scowan
 * Added members to tool.
 *
 * Revision 1.18  2008/05/09 17:49:42  scowan
 * Added shortcut key help on control panel.
 *
 * Revision 1.17  2008/04/28 23:05:09  scowan
 * Inherit from tool gps.
 *
 * Revision 1.16  2008/04/24 22:09:34  scowan
 * Control panel enabled.
 *
 * Revision 1.15  2008/04/24 17:54:55  scowan
 * Control panel initial setup.
 *
 * Revision 1.14  2008/04/23 23:06:56  scowan
 * Added auto editmode mode.
 *
 * Revision 1.13  2008/04/21 20:47:03  scowan
 * Added scroll wheel handling to allow creation of multiple lines.
 *
 * Revision 1.12  2008/02/29 23:33:26  scowan
 * Allowed support of Z values.
 *
 * Revision 1.11  2008/02/21 21:55:34  scowan
 * Added manual entry panel.
 *
 * Revision 1.10  2008/02/04 18:40:07  scowan
 * Added setVertex delegate and method to change the marker colors.
 *
 * Revision 1.9  2008/02/01 15:01:25  mju
 * Change button handlers to return bool.
 *
 * Revision 1.8  2008/01/07 18:04:45  mju
 * Add erasePoints.
 *
 * Revision 1.7  2008/01/04 18:59:27  mju
 * Generalize pointer button methods.
 *
 * Revision 1.6  2007/12/17 16:13:04  mju
 * Add setLine/List.
 *
 * Revision 1.1  2003/01/10 23:08:34  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_TOOLPOLYLINE_H
#define	INC_MGUI_TOOLPOLYLINE_H

#ifndef	INC_MGUI_TOOLGPS_H
#include <mgui/toolgps.h>
#endif

#ifndef	INC_MI32_POLYLINE_H
#include <mi32/polyline.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Polyline / polygon tool.
class TOOL_POLYLINE : public TOOL_GPS {
	public:
		//! Draw modes.
		enum DRAWMODE {
			DRAWMODE_Draw = 256,
			DRAWMODE_Stretch
			};
		
		//! Editing modes.
		enum EDITMODE {
			EDITMODE_Automatic = 0,					//!< Actual EDITMODE based on the location if the cursor to the current line
			EDITMODE_AddEnd,							//!< Add vertices to end of active line (default)
			EDITMODE_AddStart,						//!< Add vertices to start of active line
			EDITMODE_InsertVertex,					//!< Insert vertex
			EDITMODE_DragVertex,						//!< Drag nearest vertex
			EDITMODE_DeleteVertex,					//!< Delete nearest vertex
			EDITMODE_Move								//!< Move active line
			};

		//! Constructor.
		explicit TOOL_POLYLINE (
			TOOLCANVAS *canvas,						//!< View to contain tool
			bool AllowMulti = false					//!< Allow multiple lines
			);

		typedef fastdelegate::FastDelegate<bool(CHANGESEQ, EDITMODE, DPOINT3D&)> DELEGATE_ONSETVERTEX;
		typedef fastdelegate::FastDelegate<bool(POLYLINE&)> DELEGATE_ONVERTICESCHANGED;

		//! Destructor.
		virtual ~TOOL_POLYLINE ();

		//! Get reference to active line.
		//! IsActiveValid() should be used first to determine if the active line is
		//! valid.  Failure to do so will likely result in an exception.
		const POLYLINE& GetActiveLine (
			) const { return (*m_pActiveLine); }

		//! Get line editing mode.
		EDITMODE GetEditMode (
			) const { return (m_EditMode); }

		//! Get list of lines.
		const POLYLINELIST& GetLineList (
			) const { return (m_LineList); }

		//! Get number of valid lines.
		int GetNumLines (
			) const { return (m_LineList.GetNumItems()); }

		//! Determine if active line is valid.
		//! In single-line mode the active line is always valid.
		bool IsActiveValid (
			) const { return (m_pActiveLine != 0); }

		//! Determine if the lines are closed
		bool IsClosed (
			) const { return (m_IsClosed); }

		//! Lock or unlock end point of active line.
		void LockEndPoint (
			bool locked = true						//!< 'true' to lock, 'false' to unlock
			);

		//! Lock or unlock start point of active line.
		void LockStartPoint (
			bool locked = true						//!< 'true' to lock, 'false' to unlock
			);

		//! Set line to draw as "closed".
		//! This only affects how the tool is drawn, it does not add a closing point to the POLYLINE.
		void SetClosed (
			bool closed = true
			);

		//! Set delegate to call when a vertex is added or moved
		//! This allows the user of the tool to determine if the point is to be moved and where.
		//! Return 'true' for the delegate to modify the point.
		void SetDelegateOnSetVertex (
			DELEGATE_ONSETVERTEX delegate
			) { m_DelegateOnSetVertex = delegate; }

		//! Set delegate to call when a set of vertices have changed
		//! This allows the user of the tool to modify the vertex set that the user has added or altered.
		//! Return 'true' for the delegate to modify the active line.
		void SetDelegateOnVerticesChanged (
			DELEGATE_ONVERTICESCHANGED delegate
			) { m_DelegateOnVerticesChanged = delegate; }
		
		//! Set line drawing mode.
		//! Call this method AFTER the call to TOOL_BASE::CreateControlForm().
		void SetDrawMode (
			DRAWMODE DrawMode							//!< New drawing mode
			);

		//! Set line editing mode.
		void SetEditMode (
			EDITMODE EditMode							//!< New editing mode
			);

		//! Set filter distance.
		void SetFilterDistance (
			double distance
			);

		//! Set single polyline.
		void SetLine (
			const POLYLINE& polyline				//!< Polyline in tool coordinates to set
			);

		//! Set polyline list.
		void SetLineList (
			const POLYLINELIST& LineList			//!< Polyline list in tool coordinates to set
			);

		//! Set marker colors for start and end points.
		void SetMarkerColors (
			PENCOLOR startcolor,						//!< Starting point marker color setting
			PENCOLOR endcolor							//!< Ending point marker color setting
			);

		//! Set markers for start and end points.
		void SetMarkers (
			MARKER startmarker,						//!< Starting point marker type
			MARKER endmarker,							//!< Ending point marker type
			UINT16 startmarkersize = 0,			//!< Starting point marker size, 0 to retain current size
			UINT16 endmarkersize = 0				//!< Ending point marker size, 0 to retain current size
			);

		//! Update active polyline.
		void UpdateActiveLine (
			const POLYLINE& polyline				//!< Polyline in tool coordinates to update
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		enum ZONE {
			ZONE_NoFunction = 0,
			ZONE_Outside,
			ZONE_Vertex,
			ZONE_Segment,
			ZONE_StartVertex
			};

		// Tool settings
		bool m_AllowMulti;
		bool m_IsClosed;
		bool m_LockStart;
		bool m_LockEnd;
		bool m_ShowPrefiltered;
		MARKER m_startmarker;
		MARKER m_endmarker;
		UINT16 m_startmarkersize;
		UINT16 m_endmarkersize;
		PENCOLOR m_startmarkercolor;					// Starting point marker color setting
		PENCOLOR m_endmarkercolor;						// Ending point marker color setting
		EDITMODE m_EditMode;
		// Tool position values
		POLYLINELIST m_LineList;
		POLYLINE *m_pActiveLine;					// Pointer to active line
		// Values used during interaction
		ZONE m_CursorZone;						// Current cursor zone
		DPOINT3D m_prevpt;						// Previous point during button-down
		double m_MaxZoneDist;					// Maximum distance to use when searching for a non outside zone
		double m_MaxVertexDist;					// Maximum distance to use when searching for a vertex zone
		double m_FilterDistance;				// Distance in pixel units for filter
		int m_ActivePoint;
		bool m_RubberBand;
		bool m_AutoEditMode;
		INT32 m_CurVertexNum;
		INT32 m_BeginVertexNum;					// Vertex in active line corresponding to start of last drawn section
		INT32 m_EndVertexNum;					// Vertex in active line corresponding to end of last drawn section
		ZONE m_CurCursorZone;
		POLYLINE m_LastDrawnSection;				// Last drawn section in continuous tracing mode
		TRANS2D_AFFINE m_TransToolToLastDrawn;	// Transformation from tool to last drawn section coordinates

		// Delegates for TOOL_POLYLINE specific operations
		DELEGATE_ONSETVERTEX m_DelegateOnSetVertex;
		DELEGATE_ONVERTICESCHANGED m_DelegateOnVerticesChanged;
		
		// Manual Line Vertex Entry form forward declaration
		class MANUAL_POLYLINE;
		friend class MANUAL_POLYLINE;
		class FORM_POLYLINE;
		FORM_POLYLINE* m_pFormPolyLine;

		// Methods
		void AddLineIfNoActive ();
		void AddPoint (const DPOINT3D&, EDITMODE);
		void AdjustTool (DPOINT3D&, CHANGESEQ, KEYSTATE);
		bool CloseLine ();
		void ComputeCursorZone (const DPOINT2D&, bool showmarker);
		void DeletePoint (int);
		void DeletePoints (MGUI::POINT devpt);
		void DrawLastDrawnSection ();
		void DrawMarkerCursor (const DPOINT3D& markpt);
		void DrawMarkerEnd ();
		void DrawMarkerStart ();
		void DrawNeighborSegments (const LRECT2D& rectCanvas, bool SkipActive=true, int SkipSegment1=-2, int SkipSegment2=-2);
		void DrawNeighborSegments (const DRECT2D& rectTool, const POLYLINE& plTool, int SkipSegment1=-2, int SkipSegment2=-2);
		void DrawSegment (const POLYLINE&, int);
		void DrawSegment (int, bool);
		void DrawSingleLine (const POLYLINE&);
		void EraseLine ();
		void EraseMarkerCursor ();
		void EraseMarkerEnd (bool DrawSegment, bool RestoreActiveLineEnds);
		void EraseMarkerStart (bool DrawSegment, bool RestoreActiveLineEnds);
		void ErasePoint (int);
		void EraseSegment (int);
		void FilterVertices (bool FirstTime);
		void InsertPoint (int, const DPOINT3D&, UINT32);
		void MovePoint (int PointIndex, const DPOINT3D&, UINT32);
		bool PrefilterVertices (POLYLINE& PolyLine) const;
		void ResetLastDrawnSection (bool UpdateDisplay);
		bool SelectNextLine (bool SelectNext);
		void UpdateCursor () const;

		// TOOL_BASE overrides.
		virtual MGUI::FORM* v_CreateControlForm (LAYOUT_PANE_BASE& ParentPane, CTRLOPT ctrlopt);
		virtual MANUAL_BASE* v_CreateManualBase ();
		virtual void v_DoDraw ();
		virtual int v_GenerateRegion (REGION2D& region) const;
		virtual void v_OnClear ();
		virtual bool v_OnKey (MGUI::POINT point, KEYCODE keycode, KEYSTATE keystate);
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual bool v_OnPointerButtonUp (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerLeave ();
		virtual void v_OnSet3D ();
		virtual void v_OnSetTransToWindow ();

		// TOOL_GPS overrides.
		virtual void v_OnGPSCoordinate (const DPOINT3D& point);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_TOOLPOLYLINE_H
