/**
 * \file msys.h <mi32/msys.h>
 * \brief System functions
 *
 * \if NODOC
 * $Id: msys.h_v 1.24 2012/01/18 14:04:32 mju Exp $
 *
 * $Log: msys.h_v $
 * Revision 1.24  2012/01/18 14:04:32  mju
 * Remove unused msysGetLocalIP.
 *
 * Revision 1.23  2009/04/27 14:01:35  bcolombe
 * Changed path passed to MsysAppInit to pointer with null default
 *
 * Revision 1.22  2009/04/23 17:46:20  bcolombe
 * Added sessionpath to MsysAppInitialize
 * Will pass to SESSIONLOG::Initialize to use specific path
 *
 * Revision 1.21  2008/09/04 13:29:02  mju
 * More doxygen fixes.
 *
 * Revision 1.20  2008/09/03 13:19:32  mju
 * Fix doxygen warnings.
 *
 * Revision 1.19  2006/12/11 19:08:36  mju
 * Add filepath variant for msysRunAssocApp.
 *
 * Revision 1.18  2006/12/11 14:08:17  mju
 * Remove unused runProcessC fn.
 *
 * Revision 1.17  2006/07/19 12:35:48  mju
 * Add public fn to determine if lite.
 *
 * Revision 1.16  2006/05/08 16:28:08  scowan
 * *** empty log message ***
 *
 * Revision 1.15  2006/05/08 15:08:18  scowan
 * Deprecated msysRunProcess.
 *
 * Revision 1.14  2005/03/31 16:57:18  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.13  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.12  2003/06/12 15:31:19  dwilliss
 * Added CHILDPROCESS_XXX defines.  These used to be in mbfdefns.h, but that
 * file no longer exists
 *
 * Revision 1.11  2003/04/01 15:14:11  mju
 * Add MsysAppGetProductInfo.
 *
 * Revision 1.10  2003/02/11 15:30:01  scowan
 * Changed C linkage run process name.
 *
 * Revision 1.9  2002/04/10 14:43:28  dwilliss
 * Fixed so that it can still be included even if not compiling for C++
 * Yes, this is still an issue with 3rd party libs that we've patched.
 *
 * Revision 1.8  2002/01/21 15:03:46  mju
 * Add 'procname' to MsysAppInitialize().
 *
 * Revision 1.7  2002/01/17 23:51:34  mju
 * Add MsysAppInitialize.
 *
 * Revision 1.6  2002/01/10 15:47:33  mju
 * Add MsysAppOnExitFuncAdd/Remove.
 *
 * Revision 1.5  2002/01/10 14:26:20  mju
 * Use const in RunProcess.
 *
 * Revision 1.4  2002/01/09 22:04:14  mju
 * Add MsysAppCleanup.
 * Rename Get/SetAppName as AppGet/SetName.
 *
 * Revision 1.3  2002/01/09 14:19:28  mju
 * Add RunProcess prototype.
 *
 * Revision 1.2  2002/01/08 23:11:13  mju
 * Add MsysOpenReferenceDocument and MsysRunAssocApp.
 *
 * Revision 1.1  2002/01/08 18:52:03  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_MSYS_H
#define	INC_MI32_MSYS_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef __cplusplus
class MISTRING;
class FILEPATH;
#endif


//! Perform application cleanup prior to exit.
//! Calls registered OnExit functions, disconnects from RVC, etc.
void MsysAppCleanup ();

//! Get application name.
//! @return Pointer to application name.
//! Do NOT free the pointer returned.
const char* MsysAppGetName ();

#ifdef __cplusplus
//! Get descriptive information about product for use in 'Help About' dialogs.
void MsysAppGetProductInfo (
	MISTRING& string						//!< String returned.
	);
#endif

#ifdef __cplusplus
//! Initialize application.
void MsysAppInitialize (
	const char *procname,				//!< Process name for default INI and msgfile group, NULL to not set
	UINT32 ProductCode = 0,				//!< ID of product, 0 for TNTmips
	FILEPATH* sessionpath = 0			//!< Filepath for session log, empty to determine automatically
	);
#else
#ifndef GENERATING_DOXYGEN_OUTPUT
//!  version without default args for things that don't compile for C++
void MsysAppInitialize (
	const char *procname,
	const FILEPATH& sessionpath,
	UINT32 ProductCode
	);
#endif //!< GENERATING_DOXYGEN_OUTPUT
#endif

//! Determine if product is running a "lite" license.
bool MsysAppIsLite ();

#ifdef __cplusplus
//! Add function to be called on application exit during MsysAppCleanup().
//! Functions are called starting with the most recently added.
void MsysAppOnExitFuncAdd (
	void (*pfunc)(void*),				//! Pointer to function to call
	void *pdata = 0						//!< Optional data to passe to function
	);
#else
#ifndef GENERATING_DOXYGEN_OUTPUT
//!  version without default args for things that don't compile for C++
//! Add function to be called on application exit during MsysAppCleanup().
//! Functions are called starting with the most recently added.
void MsysAppOnExitFuncAdd (
	void (*pfunc)(void*),
	void *pdata
	);
#endif //!< GENERATING_DOXYGEN_OUTPUT
#endif

#ifdef __cplusplus
//! Remove previously added OnExit function.
void MsysAppOnExitFuncRemove (
	void (*pfunc)(void*),				//! Pointer to function to remove
	void *pdata = 0						//!< Optional data, must match value used in MsysAppOnExitFuncAdd
	);
#else
#ifndef GENERATING_DOXYGEN_OUTPUT
//!  version without default args for things that don't compile for C++
void MsysAppOnExitFuncRemove (
	void (*pfunc)(void*),
	void *pdata
	);
#endif //!< GENERATING_DOXYGEN_OUTPUT
#endif

//! Set application "name" for later retrieval by MsysAppGetName().
void MsysAppSetName (
	const char *newname						//!< New application name to set
	);

//! Open 'reference' document using specified tag.
//! @return Error < 0, FALSE if unrecognized tag, TRUE if valid tag.
//! Tag must be in one of the forms:
//!		'TNT:refman/file.pdf\#NamedDestination'
//!		'TNT:getstart/file.pdf?page=XXX'
//!		'GETSTART:file.pdf'
//!		'FILE:filename'
int MsysOpenReferenceDocument (
	const char *doctag
	);

//! Run application "associated" with given string / URL
//! On systems where there is no operating system-defined association system, the associations are
//!	specified by extension in the "AppAssoc" section of tnthost.ini.
ERRVALUE MsysRunAssocApp (
	const MIUNICODE *docname					//!< String / URL
	);

#ifdef __cplusplus
//! Run application "associated" with given file and pass file as parameter to it.
//! On systems where there is no operating system-defined association system, the associations are
//!	specified by extension in the "AppAssoc" section of tnthost.ini.
ERRVALUE MsysRunAssocApp (
	const FILEPATH& filepath					//!< Path to file
	);
#endif

//! Possible "result" values passed to the EndCB of MsysRunProcess
//! These used to be in mbfdefns.h, but that include file is no more.
//! Apparently Windows only uses CHILDPROCESS_Finished.  The Unix
//! platforms can get the other values
#define CHILDPROCESS_Finished				0
#define CHILDPROCESS_ExitError			1
#define CHILDPROCESS_ExternalStop		2
#define CHILDPROCESS_DiedCoreDump		3
#define CHILDPROCESS_DiedNoCoreDump		4

#ifdef __cplusplus
//! Function to execute another program.
//! \deprecated Use MISYSTEM::ExecuteProcess()
DEPRECATED int MsysRunProcess (
	const MIUNICODE *progstr, 
	void (*EndCB)(int pid, int result, void* udata) = 0,
	void *udata = 0
	);
#endif


#endif	// INC_MI32_MSYS_H
