/**
 * \file strutf8.h <mi32/strutf8.h>
 * \brief STRUTF8 class definitions.
 *
 * \if NODOC
 * $Id: strutf8.h_v 1.10 2005/09/14 14:36:31 mju Exp $
 *
 * $Log: strutf8.h_v $
 * Revision 1.10  2005/09/14 14:36:31  mju
 * Fix operator<, was not using lhs.
 *
 * Revision 1.9  2004/04/26 15:26:35  mju
 * Add GetUINT8.
 *
 * Revision 1.8  2004/02/10 16:20:03  dwilliss
 * took out operator to cast to const UINT8*.
 *
 * Revision 1.7  2004/02/10 16:10:31  dwilliss
 * Better way of implementing string comparison: One compare func and inline
 * operators to call it.
 *
 * Revision 1.6  2004/02/10 15:52:52  dwilliss
 * Added comparison operator overloads
 *
 * Revision 1.5  2004/01/21 17:20:11  dwilliss
 * fixed type-o in ifdef.
 *
 * Revision 1.4  2004/01/21 17:02:23  dwilliss
 * If building for debug and not building strutf8.c itself, define the PRIV
 * structure just enough to let VC6 and VC7.NET show the string value int
 * in a debugger watch window
 *
 * Revision 1.3  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2003/08/20 19:24:26  mju
 * Add attach, clear methods.
 *
 * Revision 1.1  2003/08/15 20:17:24  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_STRUTF8_H
#define	INC_MI32_STRUTF8_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRING;
#endif //!< GENERATING_DOXYGEN_OUTPUT

//! Container for UTF-8 encoded string.
//! Strings are reference-counted, so copying and return-by-value is very efficient.
//! Provides very minimal string modification support.  For more extensive modification
//! the MISTRING class should be used.
class CLASSLIBEXPORT STRUTF8 {
	public:

		//! Default constructor.
		STRUTF8 (
			);

		//! Copy constructor.
		STRUTF8 (
			const STRUTF8& rhs
			);

		//! Construct from null-terminated char* string.
		STRUTF8 (
			const char *rhs
			);

		//! Construct from MISTRING.
		STRUTF8 (
			const MISTRING& rhs
			);

		//! Destructor.
		~STRUTF8 ();

		//! Assignment from STRUTF8.
		STRUTF8& operator= (
			const STRUTF8& rhs
			);

		//! Assign from null-terminated char* string.
		STRUTF8& operator=  (
			const char *rhs
			);

		//! Assign from null-terminated unsigned char* string.
		STRUTF8& operator=  (
			const unsigned char *rhs
			);

		//! Assign from MISTRING.
		STRUTF8& operator=  (
			const MISTRING& rhs
			);

		//! Cast to null-terminated char* string.
		operator const char* (
			) const;

		//! Attach to char* string.
		void Attach (
			char *& str
			);

		//! Attach to unsigned char* string.
		void Attach (
			unsigned char *& str
			);

		//! Clear to empty string.
		void Clear (
			);

		//! Get UINT8 pointer to actual UTF8 buffer.
		const UINT8* GetUINT8 (
			) const;

		//! Determine if string is empty.
		//! @return true if empty, false if not.
		bool IsEmpty (
			) const;

	#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Compare to STRUTF8 string.
		bool IsEqual (
			const STRUTF8& rhs
			) const;

		//! Compare to null-terminated char* string.
		bool IsEqual (
			const char* rhs
			) const;

		//! Compare to null-terminated char* string.
		int Compare (
			const STRUTF8& rhs
			) const;

		//! Compare to null-terminated char* string.
		int Compare (
			const char* rhs
			) const;

	#endif //!< GENERATING_DOXYGEN_OUTPUT

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
			#if defined(DEBUG) && !defined(STRUTF8_C)
				// This isn't the real PRIV.  It just gives us enough to allow
				// VC6 and VC.NET debugger to show us the value of the string.
				class PRIV_DEBUG {
					const char* m_string;
					};
				PRIV_DEBUG *m_pPriv;
			#else
				class PRIV;
				PRIV *m_pPriv;
			#endif
		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

inline bool operator< (
	const STRUTF8& lhs,
	const STRUTF8& rhs
	) { return (lhs.Compare(rhs) < 0); }

inline bool operator> (
	const STRUTF8& lhs,
	const STRUTF8& rhs
	) { return (lhs.Compare(rhs) > 0); }

inline bool operator< (
	const STRUTF8& lhs,
	const char* rhs
	) { return (lhs.Compare(rhs) < 0); }

inline bool operator> (
	const STRUTF8& lhs,
	const char* rhs
	) { return (lhs.Compare(rhs) > 0); }

inline bool operator< (
	const char* lhs,
	const STRUTF8& rhs
	) { return (rhs.Compare(lhs) >= 0); }

inline bool operator> (
	const char* lhs,
	const STRUTF8& rhs
	) { return (rhs.Compare(lhs) <= 0); }

inline bool operator== (
	const STRUTF8& lhs,
	const STRUTF8& rhs
	) { return (lhs.IsEqual(rhs)); }

inline bool operator!= (
	const STRUTF8& lhs,
	const STRUTF8& rhs
	) { return (!lhs.IsEqual(rhs)); }

inline bool operator== (
	const STRUTF8& lhs,
	const char* rhs
	) { return (lhs.IsEqual(rhs)); }

inline bool operator!= (
	const STRUTF8& lhs,
	const char* rhs
	) { return (!lhs.IsEqual(rhs)); }

inline bool operator== (
	const char* lhs,
	const STRUTF8& rhs
	) { return (rhs.IsEqual(lhs)); }

inline bool operator!= (
	const char* lhs,
	const STRUTF8& rhs
	) { return (!rhs.IsEqual(lhs)); }

#undef	CLASSLIBEXPORT

#endif	//!< INC_MI32_STRUTF8_H
