/******************************************************************************
 * mi32/zipfile.h - Class for working with .zip files
 *  \if NODOC
 *  $Log: zipfile.h_v $
 *  Revision 1.10  2013/04/16 19:17:19  bcolombe
 *  Add size,filecount
 *
 *  Revision 1.9  2010/04/29 21:27:16  bcolombe
 *  Added GetNext, Find, OpenCurrent
 *
 *  Revision 1.8  2010/04/13 22:05:28  bcolombe
 *  Added includedirectory option for adddirectory
 *
 *  Revision 1.7  2010/04/07 19:43:53  bcolombe
 *  Added Unzip
 *
 *  Revision 1.6  2009/05/07 22:24:44  bcolombe
 *  Added OpenInZip
 *  This method only creates new file in zip
 *  zipfile must be open for write
 *
 *  Revision 1.5  2009/03/17 21:47:34  bcolombe
 *  AddFile now can also take the name you want the file ot have in the zipfile
 *
 *  Revision 1.4  2009/03/16 17:39:17  bcolombe
 *  AddFile and AddDirectory added
 *
 *  Revision 1.3  2007/06/08 23:56:08  ldyas
 *  added method for setting compression rating.
 *
 *  Revision 1.2  2007/06/07 14:47:16  dwilliss
 *  fixed documentation.
 *
 *  Revision 1.1  2007/02/15 18:00:18  dwilliss
 *  Initial revision
 *
 *  \endif
 **/

#ifndef INC_MI32_ZIPFILE_H
#define INC_MI32_ZIPFILE_H


#ifndef INC_MI32_GENERICRW_H
#include <mi32/genericrw.h>
#endif

#ifndef INC_MI32_FILEACCESS_H
#include <mi32/fileaccess.h>
#endif

class FILEPATH;
class MISTRING;
class MISTRINGLIST;
class FILEPATHLIST;

//! A class for reading ZIP files.
class ZIPFILE : public GENERICRW{
	public:
		ZIPFILE();
		~ZIPFILE();

		//! Add a directory and its files to a zip file
		ERRVALUE AddDirectory (
			const FILEPATH& zippath,		//!< Filepath of target zipfile
			const FILEPATH& directory,		//!< Filepath of directory containing files to zip
			bool DoSubDirectories = false,	//!< Process all sub-folders and files
			bool DoDeleteFiles = false,		//!< Delete source file after adding to zip file
			bool IncludeDirectory = false	//!< Include the directory itself in zipfile
			);

		//! Add a file to a zip file
		ERRVALUE AddFile (
			const FILEPATH& zippath,		//!< Filepath of target zipfile
			const FILEPATH& filepath,		//!< Filepath of file to add to zip
			bool DoDeleteFiles = false		//!< Delete source file after adding to zip file
			);

		//! Add a file to a zip file
		ERRVALUE AddFile (
			const FILEPATH& zippath,		//!< Filepath of target zipfile
			const FILEPATH& filepath,		//!< Filepath of file to add to zip
			const MISTRING& name,			//!< Name to give file in zip
			bool DoDeleteFiles = false		//!< Delete source file after adding to zip file
			);

		//! Get a list of files in this zip file.
		//! The file must be open in order to get the list
		ERRVALUE GetFileList (
			MISTRINGLIST& FileNameList
			);

		//! Get the global comment string for the zip file.
		//! The file must be open
		ERRVALUE GetGlobalComment (
			MISTRING& comment
			);
		
		//! Open a zip file.  This only gives access to the directory of the
		//! file.  To open a particular file within the zip file, call
		//! another method.
		ERRVALUE Open (
			const FILEPATH& filepath,
			FILE_ACCESS access = FILE_ACCESS_Read,
			FILE_SHARE share = FILE_SHARE_Any
			);

		//! Open a file within a zip file.  
		//! Note: the only access modes supported are Read and Create.
		//! There is no way to write an existing file in the zip file nor to
		//! delete one.
		ERRVALUE Open (
			const FILEPATH& filepath,
			const MISTRING& filename,		//!< Name of the file within the file
			const char* password = 0,		//!< Password if neccessary
			FILE_ACCESS access = FILE_ACCESS_Read,
			FILE_SHARE share = FILE_SHARE_Any
			);
 
		//! Open a file within a zip file.  
		//! This method requires that zipfile be already open for write
		ERRVALUE OpenInZip (
			const MISTRING& filename		//!< Name of the file within the file
			);

		//! Find a file with a zip file.
		//! Tries the next file from current pointer.  If not correct file, then locates the file by iterating through list.
		bool Find (
			const MISTRING& filename		//!< Name of the file within the file
			);

		//! Move current pointer to next file and return name of file
		MISTRING GetNext () const;

		//! Open current file
		ERRVALUE OpenCurrentFile ();

		//!Value ranges from 0(Z_NO_COMPRESSION) to 9(Z_BEST_COMPRESSION) and 1 is Z_BEST_SPEED
		void SetCompression (
			int compression = 9 //!< Most usefull usually.
			);

		//! Unzip contents to location
		ERRVALUE Unzip (
			const FILEPATH& filepath,	//!< Location of zipfile
			const FILEPATH& output,		//!< Location to unzip to
			bool bDeleteAfterUnzip		//!< Delete zipfile after unzipping
			);

		void Close ();

#ifndef GENERATING_DOXYGEN_OUTPUT
	private:
		UINT64 m_Size;
		UINT32 m_FileCount;
		class PRIV;
		PRIV *m_priv;

		virtual CAPABILITIES v_GetCapabilities() const;
		virtual INT64 v_GetSize() const;
		virtual INT64 v_GetSizeLimit() const;
		virtual INT32 v_Read(void*, INT32, bool);
		virtual ERRVALUE v_Resize(INT64);
		virtual INT64 v_Seek(INT64, SEEKFROM);
		virtual INT64 v_Tell() const;
		virtual ERRVALUE v_Write(const void*, INT32);
#endif

	};


#endif // INC_MI32_ZIPFILE_H

