/**
 * \file <rvc/autolink.h>
 * \brief RVC::AUTOLINK definitions
 *
 * \if NODOC
 * $Id: autolink.h_v 1.16 2011/05/11 20:43:26 mju Exp $
 *
 * $Log: autolink.h_v $
 * Revision 1.16  2011/05/11 20:43:26  mju
 * Add reset and setShowDlgStatus.
 *
 * Revision 1.15  2011/05/11 17:06:30  mju
 * Add issues_All.
 *
 * Revision 1.14  2011/05/11 16:31:28  mju
 * Add setDftCRS.
 *
 * Revision 1.13  2011/03/16 13:26:49  mju
 * Fix doc comments.
 *
 * Revision 1.12  2010/06/14 17:13:15  mju
 * Add clear/setAutoIssueResolution.
 *
 * Revision 1.11  2010/06/07 20:54:01  mju
 * Add ability to keep track of prior issues/resolutions.
 *
 * Revision 1.10  2010/05/12 21:06:10  mju
 * Change "optimize" to "scan".
 *
 * Revision 1.9  2010/05/05 21:13:02  mju
 * Add issue_NeedScan and corresponding method to do so.
 *
 * Revision 1.8  2010/01/27 22:17:55  bcolombe
 * Created public versions of Resolve methods that take objitem
 *
 * Revision 1.7  2008/10/17 17:39:05  scowan
 * Added region boundary resolution.
 *
 * Revision 1.6  2008/06/18 21:29:27  scowan
 * Add search tree type.
 *
 * Revision 1.2  2005/08/02 22:11:34  scowan
 * Changed to use object classes.
 *
 * Revision 1.1  2004/12/16 22:54:24  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_AUTOLINK_H
#define	INC_RVC_AUTOLINK_H

#ifndef	INC_RVC_OBJECT_H
#include <rvc/object.h>
#endif

#ifndef	GENERATING_DOXYGEN_OUTPUT
class DATETIME;
class FILEPATH;

namespace SPATREF {
	class COORDREFSYS;
	};
#endif


namespace RVC {

class AUTOLINK {
	public:

		enum ISSUES {
			ISSUE_None =				0x0000,
			ISSUE_NoCRS =				0x0001,
			ISSUE_NoPyramid =			0x0002,
			ISSUE_NoSearchTree =		0x0004,
			ISSUE_NoRegion =			0x0008,
			ISSUE_NeedScan =			0x0010,
			ISSUES_All =				0x001F
			};

		enum RESOLUTION {
			RESOLUTION_Ignore,
			RESOLUTION_FixThisFile,
			RESOLUTION_FixAllFiles
			};

		enum WARNINGS {
			WARNING_None =			0x0000,
			WARNING_NoStyles =	0x0001,
			};

		//! Reset all to normal state.
		static void Reset ();

		static void ResetPriorIssueResolution ();

		static ERRVALUE ResolveCRS (
			const RVC::OBJITEM& ObjItem,
			const SPATREF::COORDREFSYS& CoordRefSys,
			bool IgnoreErrs = false
			);

		static ERRVALUE ResolveScan (
			const RVC::OBJITEM& ObjItem,
			bool IgnoreErrs = false
			);

		static ERRVALUE ResolvePyramid (
			const RVC::OBJITEM& ObjItem,
			bool IgnoreErrs = false
			);

		static ERRVALUE ResolveRegion (
			const RVC::OBJITEM& ObjItem,
			bool IgnoreErrs = false
			);

		static ERRVALUE ResolveSearchTree (
			const RVC::OBJITEM& ObjItem,
			bool IgnoreErrs = false
			);

		static ERRVALUE ScanAndResolve (
			MDLGPARENT dlgparent,
			RVC::OBJECT& ObjFile
			);

		static ERRVALUE ScanAndResolve (
			MDLGPARENT dlgparent,
			const DATETIME& testtime,
			const FILEPATH& filepath
			);

		//! Set how to automatically resolve specified issue.
		//! User will not be prompted if this issue occurs until ClearAutoIssueResolutions is called.
		static void SetAutoIssueResolution (
			ISSUES issue,
			RESOLUTION resolution
			);

		//! Set default CRS.
		static void SetDftCRS (
			const SPATREF::COORDREFSYS& DftCRS
			);

		//! Set whether to show status dialog while resolving.
		//! Default initially and after Reset is to show.
		static void SetShowDlgStatus (
			bool ShowDlgStatus
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class ISSUEITEM;
		class DLG;

		typedef MILIST<ISSUEITEM> ISSUEITEMLIST;

		static ISSUES GetPriorIssuesResolved ();

		static RESOLUTION GetPriorIssueResolution (ISSUES issue);

		static ERRVALUE ResolveCRS (
			const ISSUEITEMLIST& IssueItemList,
			const SPATREF::COORDREFSYS& CoordRefSys,
			bool IgnoreErrs
			);

		static ERRVALUE ResolvePyramid (
			const ISSUEITEMLIST& IssueItemList,
			bool IgnoreErrs
			);

		static ERRVALUE ResolveScan (
			const ISSUEITEMLIST& IssueItemList,
			bool IgnoreErrs
			);

		static ERRVALUE ResolveRegion (
			const ISSUEITEMLIST& IssueItemList,
			bool IgnoreErrs
			);

		static ERRVALUE ResolveSearchTree (
			const ISSUEITEMLIST& IssueItemList,
			bool IgnoreErrs
			);

		static void ScanChildren (
			const RVC::OBJECT& ObjParent,
			ISSUEITEMLIST& IssueItemList,
			WARNINGS& warnings
			);

		static void ScanFolderFiles (
			const FILEPATH& filepath,
			ISSUEITEMLIST& IssueItemList
			);

		static void SetPriorIssueResolution (ISSUES issue, RESOLUTION resolution);

		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(RVC::AUTOLINK::ISSUES)
DEFINE_ENUM_OP_BITWISE(RVC::AUTOLINK::WARNINGS)
#endif

}	// End namespace RVC

#endif	// INC_RVC_AUTOLINK_H
