/**
 * \file rvc/georegion.h 
 * \brief interface for the RVC::GEOREGION class.
 *
 * \if NODOC
 * $Id: georegion.h_v 1.21 2012/10/09 15:00:34 mju Exp $
 *
 * $Log: georegion.h_v $
 * Revision 1.21  2012/10/09 15:00:34  mju
 * Add "separate" to setupElemTransList.
 *
 * Revision 1.20  2010/04/23 13:05:31  mju
 * Add v_findElemSet.
 *
 * Revision 1.19  2007/10/03 20:36:56  scowan
 * Changed name of geomregion to georegion.
 *
 * Revision 1.18  2007/10/03 17:56:21  scowan
 * Added virtual methods to handle lines and points.
 *
 * Revision 1.17  2007/04/13 15:17:44  scowan
 * Added readRegion virtual method override.
 *
 * Revision 1.16  2006/12/29 21:16:17  scowan
 * Added readElement virtual methods.
 *
 * Revision 1.15  2006/06/27 15:04:15  scowan
 * Make method now fills in objitem with created object info.
 *
 * Revision 1.14  2005/12/21 23:09:07  scowan
 * Added support for the readNext virtual methods.
 *
 * Revision 1.13  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.12  2005/12/09 17:29:31  scowan
 * Updated to use credentials.
 *
 * Revision 1.11  2005/10/20 17:01:05  scowan
 * Quadtree and geometric interface changes supported.
 *
 * Revision 1.10  2005/01/19 21:20:52  scowan
 * Added support for descriptor.
 *
 * Revision 1.9  2004/12/18 00:08:08  scowan
 *  Added component field to findClosestElements.
 *
 * Revision 1.8  2004/08/24 22:26:36  scowan
 * Added virtual methods to get and set total line length.
 *
 * Revision 1.7  2004/07/12 23:21:47  scowan
 * Removed use of geo region.
 *
 * Revision 1.6  2004/06/25 16:12:08  scowan
 * Uses correct region class.
 *
 * Revision 1.5  2004/05/28 20:36:10  scowan
 * Added new virtual method override.
 *
 * Revision 1.4  2004/05/28 13:51:20  scowan
 * Changed conversion code to fit the pattern.
 *
 * Revision 1.3  2004/05/27 21:29:47  scowan
 * Fixed cad and added region to extract to.
 *
 * Revision 1.2  2004/03/22 16:26:37  scowan
 * Support find closest elements virtual method.
 *
 * Revision 1.1  2004/03/15 21:55:24  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::GEOREGION rvc/georegion.h
//!	\brief Manages RVC::OBJTYPE_Region RVC objects as GEOMETRIC objects.
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Region.
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Region.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
//!		- RVC::OBJTYPE_Vector
//!		- RVC::OBJTYPE_TIN
//!		- RVC::OBJTYPE_CAD
//!		- RVC::OBJTYPE_Raster
//!		- RVC::OBJTYPE_RasterSet
//!		- RVC::OBJTYPE_Hyperspectral

#ifndef INC_RVC_GEOREGION_H
#define INC_RVC_GEOREGION_H

#ifndef INC_RVC_GEOMETRC_H
	#include <rvc/geometrc.h>
#endif

#ifndef INC_MI32_REGION2D_H
	#include <mi32/region2d.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace SPATREF {
	class COORDREFSYS;
	}
#endif // GENERATING_DOXYGEN_OUTPUT

namespace RVC {

class GEOREGION : public GEOMETRIC {
	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class MAKEPARMS;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
		//! Default constructor
		GEOREGION (
			);

		//! Copy constructor
		GEOREGION (
			const GEOREGION& rhs
			);

		//! Destructor
		virtual ~GEOREGION (
			);

		//! Assignment, will close and re-open object if possible
		GEOREGION& operator= (
			const GEOREGION& rhs
			);

		//! Create new shape object given parent object, name, description, creation parameters, and source
		ERRVALUE Make (
			const OBJECT& parent,
			const DESCRIPTOR& Descriptor,
			const MAKEPARMS& MakeParms,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);
		
		//! Create new shape object given OBJITEM, creation parameters, and source
		ERRVALUE Make (
	    	OBJITEM& ObjItem,													//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const MAKEPARMS& MakeParms,
			const char* Source = 0,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Get the current region georeference from this instance
		const SPATREF::COORDREFSYS& GetCoordRefSys (
			) const;
			
		//! Get the current region from this instance
		const REGION2D& GetRegion (
			) const;
			
		//! Write current region out to file			
		ERRVALUE Write (
			);
			
		//! Set the current region from this instance
		void SetRegion (
			const REGION2D& region
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);

		//! Implementation of GEOSPATIAL virtual methods
		virtual void v_GetExtents (DRECT3D &extents) const;
		virtual void v_GetScale (double& xscale, double& yscale) const;
		virtual double v_GetZOffset () const;
		virtual double v_GetZScale () const;
		virtual bool v_HasCapability (int Capability) const;
		virtual void v_SetScale (double xscale, double yscale);
		virtual void v_SetZOffset (double zoffset);
		virtual void v_SetZScale (double zscale);

		//! Implementation of GEOMETRIC pure virtual methods
		virtual ERRVALUE v_ExtractTo (GEOMETRIC& DestObj, GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		virtual INT32 v_FindClosestElementLow (ELEMTYPE ElemType, const DPOINT2D& SearchPt, GEOMETRIC::SEARCHFILTER* SearchFilter, double SearchDist, double* DistFromElem) const;
		virtual ERRVALUE v_GetElementExtents (ELEMTYPE ElemType, INT32 ElemNum, DRECT3D& Extents) const;
		virtual INT32 v_GetNextElementID (ELEMTYPE ElemType) const;
		virtual INT32 v_GetNumElements (ELEMTYPE ElemType) const;
		virtual POINTTYPE v_GetPointType () const;
		virtual INT32 v_GetPointSize () const;
		virtual QSEARCHTREE* v_GetSearchTree (ELEMTYPE ElemType) const;
		virtual double v_GetTotalLineLength () const;
		virtual bool v_IsElementDeleted (const ELEMENT& Element) const;
		virtual bool v_IsEmpty () const;
		virtual ERRVALUE v_SetExtents (const DRECT3D &extents);
		virtual ERRVALUE v_SetNextElementID (ELEMTYPE ElemType, INT32 NextElementID);
		virtual void v_SetSearchTree (ELEMTYPE ElemType, QSEARCHTREE* SearchTree) const;
		virtual ERRVALUE v_SetTotalLineLength (double length);
		virtual int v_SetupElemTransList (const GEOMETRIC::ELEMSELECTPARMS& SelectParms, GEOMETRIC::ELEMTRANSLIST& TransList, GEOMETRIC::ELEMSELECTFILTER *SelectFilter, bool separate) const;
		virtual bool v_TestElementAgainstRegion (const ELEMENT& Element, const REGION2D& region, TESTCRITERIA TestMode) const;
		virtual ERRVALUE v_UpdateExtents (const DRECT3D &extents);

		//! Implementation of GEOMETRIC virtual methods
		virtual ERRVALUE v_FindClosestElementsLow (const DPOINT2D& SearchPt, double SearchDist, SEARCHFILTER* SearchFilter, SIMPLE_ARRAY<ELEMENT>& ElementList, COMPONENT Component) const;
		virtual ERRVALUE v_FindElementDrawSet (ELEMTYPE ElemType, const DRECT2D& SearchExtents, SEARCHRESULT& SearchResult) const;
		virtual ERRVALUE v_FindElementSearchSet (ELEMTYPE ElemType, const DRECT2D& SearchExtents, SEARCHRESULT& SearchResult) const;
		virtual ERRVALUE v_FindElementSet (ELEMTYPE ElemType, const DRECT2D& SearchExtents, UNSIGNEDSET& ElemSet, bool BuildIndex) const;
		virtual INT32 v_GetAttachmentIndex (const ELEMENT& Element) const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, POLYLINELIST& LineList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, POLYLINE& PointList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, REGION2D& Region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextLine (ELEMENT& Element, POLYLINELIST& LineList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextPoint (ELEMENT& Element, POLYLINE& PointList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextRegion (ELEMENT& Element, REGION2D& Region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadRegion (const BITSET_UNOWNED& ElemSet, REGION2D& Region, DIMENSION dim, bool NoIslands) const;
		
		POINTTYPE DeterminePointType () const;
 		ERRVALUE ExtractToVector (GEOMETRIC& DestObj, const ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractToCAD (GEOMETRIC& DestObj, const ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractToRegion (GEOMETRIC& DestObj, const ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		
		REGION2D m_Region;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	

class GEOREGION::MAKEPARMS {
	public:
		MAKEPARMS (
			const REGION2D& region
			) :
			m_Region(region)
			{}
	
	private:
		REGION2D m_Region;
		
		friend class GEOREGION;			
	};
	
}		//! End of RVC namespace

#endif 	//!< INC_RVC_GEOREGION_H
