/******************************************************************************
 *
 * \file sml/rasterobject.h
 * \brief SML RASTEROBJECT
 *
 * \if NODOC
 * $Log: rasterobject.h_v $
 * Revision 1.4  2007/11/12 16:11:40  scowan
 * Removed legacy image statistics handling methods.
 *
 * Revision 1.3  2007/11/09 17:54:37  dwilliss
 * got rid of some unused members
 * Commented some methods that can go away after next full build
 *
 * Revision 1.2  2007/05/08 15:46:16  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 21:07:17  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_RASTEROBJECT_H
#define INC_SML_RASTEROBJECT_H

#ifndef INC_SML_OBJECT_H
#include <sml/object.h>
#endif

#ifndef INC_RVC_IMAGE_H
#include <rvc/image.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class RASTER;
	class RASTERHANDLE;
	}
#endif

namespace SML {
#ifndef GENERATING_DOXYGEN_OUTPUT
class RASTBUF;
#endif

/*****************************************************************************/
/**
*
*	\note ctor/dtor are private and only called by friend SML::CONTEXT
*/
class RVCSYSLIBCLASSEXPORT RASTEROBJECT : public OBJECT {
	public:

		//! Get current line
		int GetCurLin (
			) const;

		//! Get current column
		int GetCurCol (
			) const;

		//! Get a pointer to the RVC::RASTER interface class.
		//! Do NOT free this pointer.  It will be destroyed when the SML::OBJECT is
		//! destroyed.  The interface returned is Attached to the object but may
		//! or may not be open.
		//! If the SML::BOJECT does not refer to any existing real object, this
		//! method will return NULL.
		RVC::RASTER* GetInterface (
			) const;

		//! Returns true if the Null value is known
		//! (should be named IsNullKnown)
		bool GetNullKnown (
			) const;

		//! Return the raster's NULL value as a double
		//!  return value undefined if the raster doesn't have a NULL value.
		double GetNullValue (
			) const;

		//! Returns the number of bits/cell
		DEPRECATED int GetNumBits (
			) const;

		//! Returns the number of raster columns
		int GetNumCols (
			) const;

		//! Returns the number of raster lines
		int GetNumLins (
			) const;

		//! Returns true if the NULL cells are being treated as NULL
		//! Returns false if NULL cells are being treated as normal values
		bool GetUseNull (
			) const;

		//! Get the data type (RVCTYPE_Xxx)
		DEPRECATED int GetDataType (
			) const;

		//! Force a re-read of the histogram the next time GetImageStats() is called.
		void InvalidateImageStats (
			);

		//! Set the current column (used by foreach loops)
		void SetCurCol (
			int col
			);

		//! Set the current line (used by foreach loops)
		void SetCurLin (
			int lin
			);

		// This method should go away after 10 Nov 07 -- nothing calles it now.
		//! Set whether or not histogram has been read.
		void SetHistoRead (
			bool known
			);

		//! Set a pointer to the RVC::RASTER interface class.
		//! After setting the interface, the SML::OBJECT "owns" it and will 
		//! delete it in its own destructor.  Do not under any circumstances
		//! pass a pointer to an local variable that will go out of scope.
		//! Since SML::OBJECT is implemented in rvcsys.dll it does not have
		//! have access to the copy constructor.
		void SetInterface (
			RVC::RASTER* obj
			);

		void SetMaxValue (
			double val
			);

		void SetMinValue (
			double val
			);

		void SetNullKnown (
			bool known
			);

		void SetNullValue (
			double val
			);

		void SetUseNull (
			bool use
			);

		void SetupRaster (
			);

		//! Flush the internal buffers
		int Flush (
			);

		//! Reads the cell value at a given location
		//! Sets ErrorCode if fatal error.
		double ReadCell (
			int lin,
			int col
			);

		//! Read a single raster cell
		int ReadCell (
			int lin,
			int col,
			double* pval
			);

		//! Write a single raster cell
		int WriteCell (
			int lin,
			int col,
			double val
			);

		//! Write a span of raster cells all the same value
		int WriteSpan (
			int lin,
			int startcol,
			int endcol,
			double val
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Constructor
		RASTEROBJECT (
			CONTEXT* context
			);

		//! Destructor
		virtual ~RASTEROBJECT (
			);

		virtual int v_OpenLow (
			int mode
			);

		virtual void v_CloseLow (
			);

		virtual RVC::OBJTYPE v_GetObjType (
			) const;

		int AddBuffer (
			);

		int FindBuffer (
			int lin,
			int col,
			UINT32 flags,
			RASTBUF** pbuf
			);

		RVC::RASTERHANDLE* GetHandle (
			) const { return m_handle; }

		INT32 m_numlins;
		INT32 m_numcols;
		INT32 m_TileCols;
		INT32 m_datatype;
		INT32 m_numbits;

		RASTBUF *m_pBufList;
		int m_hit, m_miss, m_numbufs;
		double m_nullval;
		double m_min, m_max;
		bool m_UseNull;
		bool m_bUserSetUseNull;
		bool m_NullKnown;
		RVC::RASTERHANDLE* m_handle;
		RVC::RASTERHANDLE* m_NullMaskHandle;

		friend class CONTEXT;		//!< Only SML::CONTEXT can call our ctor/dtor
		friend class OBJECT;			//!< All our virtual methods are private and called through SML::OBJECT
		friend class VM;			//!< Virtual Machine needs access to otherwise private methods
#endif // GENERATING_DOXYGEN_OUTPUT
   };
} // end of SML namespace


typedef SML::RASTEROBJECT SMLRASTEROBJECT;

/*****************************************************************************/

#define QQCurRastSymLin(sym) ((sym)->m_obj->GetCurElemNum((ELEMTYPE)0))
#define QQCurRastSymCol(sym) ((sym)->m_obj->GetCurElemNum((ELEMTYPE)1))


#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		//  INC_SML_RASTEROBJECT_H


