/**
 * \file imagesample.h <spatmod/imagesample.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SAMPLE class definition
 *
 * \if NODOC
 * $Id: imagesample.h_v 1.4 2007/09/20 14:07:11 mju Exp $
 *
 * $Log: imagesample.h_v $
 * Revision 1.4  2007/09/20 14:07:11  mju
 * Add type_Validity.
 *
 * Revision 1.3  2007/01/02 18:54:54  mju
 * Add exchange() method.
 *
 * Revision 1.2  2006/11/21 17:43:47  mju
 * Add setType.
 *
 * Revision 1.1  2005/08/16 15:10:45  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESAMPLE_H
#define	INC_SPATMOD_IMAGESAMPLE_H

#ifndef	INC_SPATMOD_DATATYPE_H
#include <spatmod/datatype.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Define properties of a single sample.
//!  Properties include sample type, data type, value and optional precision.
//!  A sample may be attached to an external data buffer for accessing values.
//!  A set of one or more SAMPLE instances is used to represent a PIXEL.
class SAMPLE {

	public:

		//! Enumeration of what a sample represents.
		enum TYPE {
			TYPE_Invalid =				0,
			TYPE_Unknown =				1,
			TYPE_Index =				2,		//!< Index into palette or other array
			TYPE_Gray =					3,
			TYPE_Red =					4,
			TYPE_Green =				5,
			TYPE_Blue =					6,
			TYPE_Opacity =				7,
			TYPE_Cyan =					8,
			TYPE_Magenta =				9,
			TYPE_Yellow =				10,
			TYPE_Black =				11,
			TYPE_Hue =					12,
			TYPE_Saturation =			13,
			TYPE_ComplexMagnitude =	14,	//!< Magnitude component of complex value
			TYPE_ComplexPhase =		15,	//!< Phase component of complex value
			TYPE_ComplexReal =		16,	//!< Real component of complex value
			TYPE_ComplexImaginary =	17,	//!< Imaginary component of complex value
			TYPE_Validity =			18,	//!< Used as validity for following samples
			};

		//! Default constructor.
		SAMPLE ();

		//! Copy constructor.
		SAMPLE (
			const SAMPLE& rhs
			);

		//! Construct sample with specified properties.
		SAMPLE (
			TYPE SampleType,					//!< Sample type
			DATATYPE DataType,				//!< Data type
			const void* pValue = 0			//!< Optional pointer to initial sample value
			);

		//! Destructor.
		~SAMPLE ();

		//! Assignment.
		SAMPLE& operator= (
			const SAMPLE& rhs
			);

		//! Compare sample values.
		//! Handles all appropriate data type conversions.
		//! @return -1 if this value < rhs, 0 if values equal, +1 if this > rhs.
		int CompareValues (
			const SAMPLE& rhs					//! Sample to compare to
			) const;

		//! Exchange data with another SAMPLE.
		void Exchange (
			SAMPLE& other						//!< Sample to exchange with
			);

		//! Get DATATYPE of sample.
		DATATYPE GetDataType (
			) const { return (m_DataType); }

		//! Get number of bytes in sample.
		UINT32 GetNumBytes (
			) const { return (m_DataType & DATATYPE_MASK_Size); }

		//! Get precision of sample.
		//! @return Number of bits of precision.
		UINT16 GetPrecision (
			) const { return (m_Precision); }

		//! Get SAMPLE::TYPE of sample.
		TYPE GetType (
			) const { return (m_SampleType); }

		//! Get value as double.
		//! Conversion is done by casting, may lose precision for 64-bit integer values.
		double GetValueDouble (
			) const;

		//! Get value as INT32.
		//! Conversion is done by casting, may overflow for out of range values.
		INT32 GetValueINT32 (
			) const;

		//! Get pointer to sample's value.
		//! Caller must cast returned pointer to appropriate type before using.
		const void* GetValuePtr (
			) const { return (m_pValue); }

		//! Set data type of sample.
		//! Value will be reset to 0 and precision will be reset to default.
		void SetDataType (
			DATATYPE DataType
			);

		//! Set precision of sample.
		//! By default the precision is equal to the number of bits in the sample's DATATYPE.
		//! Precision may not exceed number of bits in sample's DATATYPE.
		//! Supported for integer data types up to 32 bits only.
		//! Data value is assumed to occupy the least significant bit(s).
		void SetPrecision (
			int precision
			);

		//! Set sample type.
		void SetType (
			TYPE SampleType
			) { m_SampleType = SampleType; }

		//! Set sample value from double.
		//! No range checking is performed.
		void SetValue (
			double value							//!< Value to set
			);

		//! Set sample value using pointer to source value.
		//! The source pointer must point to a value of the sample's datatype, with proper
		//! memory alignment if required by processor.
		void SetValueFromPtr (
			const void *pValue					//!< Pointer to value
			);

		//! Set sample value to maximum for DATATYPE and current precision.
		void SetValueMaximum (
			);

		//! Set sample value to minimum for DATATYPE and current precision.
		void SetValueMinimum (
			);

		//! Set sample value to "not a number" for float data types.
		//! Has no effect for integer data types.
		void SetValueNAN (
			);

		//! Set sample value to 0.
		void SetValueZero (
			);

		//! Compare samples for equality of sample type, data type and value.
		bool IsEqual (
			const SAMPLE& rhs
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		void *m_pValue;
		TYPE m_SampleType;
		DATATYPE m_DataType;
		UINT16 m_Precision;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//! Compare samples for equality of sample type, data type and value.
inline bool operator== (
	const SAMPLE& lhs,
	const SAMPLE& rhs
	) { return (lhs.IsEqual(rhs)); }

//! Compare samples for inequality of sample type, data type or value.
inline bool operator!= (
	const SAMPLE& lhs,
	const SAMPLE& rhs
	) { return (!lhs.IsEqual(rhs)); }

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	 // INC_SPATMOD_IMAGESAMPLE_H
