/**
 * \file imagetarget.h <spatmod/imagetarget.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET definition.
 *
 * \if NODOC
 * $Id: imagetarget.h_v 1.10 2011/03/31 21:36:09 mju Exp $
 *
 * $Log: imagetarget.h_v $
 * Revision 1.10  2011/03/31 21:36:09  mju
 * Add v_logCreate.
 *
 * Revision 1.9  2011/02/03 21:26:03  mju
 * Change v_getGeoreference to use zoom.
 *
 * Revision 1.8  2008/08/08 18:17:16  mju
 * Fix doc comment.
 *
 * Revision 1.7  2008/08/08 18:00:31  mju
 * Add setZeroInvalidSamples.
 *
 * Revision 1.6  2007/08/30 15:45:40  mju
 * Add process variant with iterator.
 * Add setupIterator methods.
 *
 * Revision 1.5  2006/12/11 14:34:12  mju
 * Make pushBegin/End/SubSection public and inline.
 *
 * Revision 1.4  2006/11/28 16:40:35  mju
 * Add parm to process() for whether to show message.
 *
 * Revision 1.3  2005/12/16 16:07:41  mju
 * Add v_getGeoreference.
 *
 * Revision 1.2  2005/08/30 12:22:38  mju
 * Add docs.
 *
 * Revision 1.1  2005/08/16 15:11:09  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGET_H
#define	INC_SPATMOD_IMAGETARGET_H

#ifndef	INC_SPATMOD_IMAGEBASE_H
#include <spatmod/imagebase.h>
#endif

#ifndef INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class STAGE;
class SECTION;
class SECTION_ITERATOR;
class BUFFER;

//=====================================================================================================================

//! Base for pipeline target.
class TARGET : public BASE {
	public:

		//! Standard constructor.
		//! Target does not take ownership of input stage.
		TARGET (
			STAGE *pInputStage					//!< Stage in pipeline to obtain image data from
			);

		//! Destructor.
		virtual ~TARGET ();

		//! Initialize target.
		//! Must be called after construction and before Process().
		ERRVALUE Initialize (
			);

		//! Process entire image.
		ERRVALUE Process (
			bool ShowMsg = true					//!< Show "processing" message at appropriate time
			);

		//! Process section of image.
		ERRVALUE Process (
			const SECTION& section,				//!< Section to process
			bool ShowMsg = true					//!< Show "processing" message at appropriate time
			);

		//! Process using specified iterator.
		//! Iterator does not need to be at beginning of section, this is useful for continuing partially complete jobs.
		ERRVALUE Process (
			const SECTION_ITERATOR& iterator,	//!< Iterator
			bool ShowMsg = true						//!< Show "processing" message at appropriate time
			);

		//! Begin push into target.
		//! Called by Process, not usually called directly.
		ERRVALUE PushBegin (
			const SECTION_ITERATOR& iterator
			) { return (v_PushBegin(iterator)); }

		//! End push into target.
		//! Called by Process, not usually called directly.
		ERRVALUE PushEnd (
			) { return (v_PushEnd()); }

		//! Push subsection to target.
		//! Called by Process, not usually called directly.
		ERRVALUE PushSubSection (
			const SECTION& SubSection,
			const BUFFER& buffer
			) { return (v_PushSubSection(SubSection,buffer)); }

		//! Setup iterator for entire image.
		ERRVALUE SetupIterator (
			SECTION_ITERATOR& iterator			//!< Iterator returned
			) const;

		//! Setup iterator for specified section.
		//! If section extends outside image extent, only portion inside will be used.
		ERRVALUE SetupIterator (
			const SECTION& section,				//!< Section
			SECTION_ITERATOR& iterator			//!< Iterator returned
			) const;

		//! Set whether to zero invalid samples before writing.
		//! Default is to not do so.
		void SetZeroInvalidSamples (
			bool ZeroInvalidSamples
			) { m_ZeroInvalidSamples = ZeroInvalidSamples; }

	protected:

		//! Get pointer to input stage.
		IMAGE::STAGE* GetInputStage (
			) const { return (m_pInputStage); }

		//! Set Y size of strip to process.
		//! If not set will use default determined by tile size and processing order capabilities.
		void SetStripSizeY (
			double StripSizeY						//!< Size of strip in rows
			);

		//! Get pixel lookup table.
		//! Default implementation returns lookup table from input stage if any
		virtual const PIXELTABLE* v_GetPixelLookupTable (
			) const;

		//! Query image for specified capability.
		//! Default returns true for CAPABILITY_Push and false for all others.
		virtual bool v_QueryCapability (
			CAPABILITY capability				//!< Capability to query
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		IMAGE::STAGE *m_pInputStage;
		double m_StripSizeY;
		bool m_ZeroInvalidSamples;

		ERRVALUE ProcessLow (
			SECTION_ITERATOR& iterator,
			bool ShowMsg
			);

		ERRVALUE SetTotalSection (
			const SECTION& TotalSection
			);

		// BASE overrides.
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_LogCreate (const char *label, bool OnlyEnds, MILOG::MODE mode);

		// Unimplemented
		TARGET (const TARGET&);
		TARGET& operator= (const TARGET&);
		#endif // GENERATING_DOXYGEN_OUTPUT

		// TARGET overridables

		//! Initialize target.
		//! Must do the following (in order), checking for error after each call.
		//! Validate all class-specific settings.
		//! Open any files if required.
		virtual ERRVALUE v_Initialize (
			) = 0;

		//! Begin push of a specific section.
		//! Usually only called once at start of Process() operation.
		virtual ERRVALUE v_PushBegin (
			const SECTION_ITERATOR& iterator		//!< SubSection iterator to be used
			) = 0;

		//! Finish pushing subsections.
		//! Usually only called once at end of Process() operation.
		//! If any extra processing is done it should be done the first time this is called.
		virtual ERRVALUE v_PushEnd (
			) = 0;

		//! Push data for specified subsection from buffer.
		virtual ERRVALUE v_PushSubSection (
			const SECTION& SubSection,				//!< Current subsection to write
			const BUFFER& buffer						//!< Buffer to write from
			) = 0;

		//! Set total section of image to process for job.
		//! Default implementation does nothing.
		virtual ERRVALUE v_SetTotalSection (
			const SECTION& TotalSection
			);

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGET_H

